@@ -1,9 +1,9 @@
 # Copyright 2024 Marimo. All rights reserved.
 from __future__ import annotations
 
-import ast
 import base64
 import inspect
+import os
 import sys
 import threading
 from collections.abc import Iterable, Iterator, Mapping
@@ -24,6 +24,7 @@ from typing import (
 
 from marimo._ast.app_config import _AppConfig
 from marimo._ast.cell_id import external_prefix
+from marimo._ast.parse import ast_parse
 from marimo._ast.variables import BUILTINS
 from marimo._convert.converters import MarimoConvert
 from marimo._schemas.serialization import (
@@ -76,7 +77,7 @@ if TYPE_CHECKING:
 P = ParamSpec("P")
 R = TypeVar("R")
 Fn: TypeAlias = Callable[P, R]
-Cls: TypeAlias = type
+Cls = TypeVar("Cls", bound=type)
 LOGGER = _loggers.marimo_logger()
 
 
@@ -530,7 +531,7 @@ class App:
             errors: list[str] = []
             for code in self._unparsable_code:
                 try:
-                    ast.parse(dedent(code))
+                    ast_parse(dedent(code))
                 except SyntaxError as e:
                     error_line = e.text
                     error_marker: str = (
@@ -593,11 +594,133 @@ class App:
 
     def run(
         self,
+        defs: dict[str, Any] | None = None,
+        **kwargs: Any,
     ) -> tuple[Sequence[Any], Mapping[str, Any]]:
-        self._maybe_initialize()
+        """
+        Run the marimo app and return its outputs and definitions.
+
+        Use this method to run marimo apps programmatically and retrieve their
+        outputs and definitions. This lets you execute notebooks from other
+        Python scripts. By providing definitions to `app.run()`, you can
+        override specific cells in the notebook with your own values.
+
+
+        Examples:
+            Consider a notebook `my_notebook.py`:
+
+            ```python
+            import marimo
+
+            app = marimo.App()
+
+            with app.setup:
+                import pandas as pd
+
+
+            @app.cell
+            def config():
+                batch_size = 32
+                learning_rate = 0.01
+                return batch_size, learning_rate
+
+
+            @app.cell
+            def process_data(pd, batch_size, learning_rate):
+                data = pd.DataFrame({"x": [1, 2, 3]})
+                result = data * batch_size * learning_rate
+                return (result,)
+
+
+            if __name__ == "__main__":
+                app.run()
+            ```
+
+            To run this app programmatically:
+
+            ```python
+            from my_notebook import app
+
+            # Run with default values
+            outputs, defs = app.run()
+            # defs["batch_size"] == 32, defs["learning_rate"] == 0.01
+
+            # Override the specific cell definitions in `config`
+            outputs, defs = app.run(
+                defs={batch_size: 64, learning_rate: 0.001}
+            )
+            # defs["batch_size"] == 64, defs["learning_rate"] == 0.001
+            ```
+
+        Definition Override Behavior:
+            When you provide definitions to `app.run()`, you are **completely
+            overriding** the definitions of cells that define those variables:
+
+            - The cells that originally defined those variables will not execute
+            - You must provide **all** the definitions that a cell would normally produce
+            - Cells that depend on the overridden variables will use your provided values
+
+        Args:
+            defs (dict[str, Any]):
+                You may pass values for any variable definitions as keyword
+                arguments. marimo will use these values instead of executing
+                the cells that would normally define them. Cells that depend
+                on these variables will use your provided values.
+            **kwargs (Any):
+                For forward-compatibility with future arguments.
+
+        Returns:
+            A tuple containing:
+            - Sequence of cell outputs (visual outputs from each cell)
+            - Mapping of variable names to their values (definitions)
+
+        Environment Variables:
+            MARIMO_SCRIPT_EDIT: If set, opens the notebook in edit mode instead
+                of running it. Requires the app to have a filename.
+        """
+        del kwargs
+        # Enabled specifically for debugging purposes.
+        # see docs.marimo.io/guides/debugging
+        if os.environ.get("MARIMO_SCRIPT_EDIT"):
+            # unset the env var to avoid recursion
+            os.environ.pop("MARIMO_SCRIPT_EDIT")
+            from marimo._cli.cli import edit
+
+            if self._filename is None:
+                raise RuntimeError(
+                    "MARIMO_SCRIPT_EDIT is set, but filename cannot be determined."
+                )
+            ctx = edit.make_context("edit", ["--watch", self._filename])
+            edit.invoke(ctx)
+            return ((), {})
+
+        try:
+            self._maybe_initialize()
+        except (CycleError, MultipleDefinitionError, UnparsableError) as e:
+            from marimo._lint import collect_messages
+
+            if self._filename is not None:
+                # Run linting checks to provide better error messages for breaking errors.
+                linter, messages = collect_messages(self._filename)
+                if messages:
+                    sys.stderr.write(messages)
+                # Re-raise the original exception but without trace
+                marimo_error = type(e)(str(e))
+                raise marimo_error from None
+            else:
+                raise
+
         glbls: dict[str, Any] = {}
         if self._setup is not None:
-            glbls = self._setup._glbls
+            glbls = {**self._setup._glbls}
+
+        if set(glbls) & set(defs or {}):
+            # Type Error is convention for bad args.
+            raise TypeError("`defs` cannot override setup cell definitions.")
+
+        if defs is not None:
+            glbls.update(defs)
+
         outputs, glbls = AppScriptRunner(
             InternalApp(self),
             filename=self._filename,
