@@ -5,14 +5,28 @@ from marimo._ast import codegen
 
 
 def markdown_to_marimo(source: str) -> str:
+    # NB. This should be kept in sync with the logic in
+    # frontend/src/core/codemirror/language/languages/markdown.ts
+    # ::transformOut
     source = source.replace('"""', '\\"\\"\\"')
+
+    # 6 quotes in a row breaks
+    if not source:
+        source = " "
+
+    # If quotes are on either side, we need to use the multiline format.
+    bounded_by_quotes = source.startswith('"') or source.endswith('"')
+
+    if "\n" not in source and not bounded_by_quotes:
+        return f'mo.md(r"""{source}""")'
+
     return "\n".join(
         [
             "mo.md(",
             # r-string: a backslash is just a backslash!
             codegen.indent_text('r"""'),
-            codegen.indent_text(source),
-            codegen.indent_text('"""'),
+            source,
+            '"""',
             ")",
         ]
     )
