@@ -1,4 +1,5 @@
 # Copyright 2024 Marimo. All rights reserved.
+
 from __future__ import annotations
 
 import datetime
@@ -14,8 +15,8 @@ from typing import (
     cast,
 )
 
-import narwhals.stable.v1 as nw
-from narwhals.typing import IntoDataFrame
+import narwhals.stable.v2 as nw
+from narwhals.typing import IntoDataFrame, IntoLazyFrame
 
 from marimo import _loggers
 from marimo._dependencies.dependencies import DependencyManager
@@ -29,6 +30,7 @@ from marimo._utils.narwhals_utils import (
     assert_can_narwhalify,
     can_narwhalify,
     empty_df,
+    is_narwhals_lazyframe,
 )
 
 if sys.version_info < (3, 10):
@@ -54,7 +56,9 @@ VegaSpec = dict[str, Any]
 RowOrientedData = list[dict[str, Any]]
 ColumnOrientedData = dict[str, list[Any]]
 
-ChartDataType = Union[IntoDataFrame, RowOrientedData, ColumnOrientedData]
+ChartDataType = Union[
+    IntoDataFrame, IntoLazyFrame, RowOrientedData, ColumnOrientedData
+]
 
 # Union of all possible chart types
 AltairChartType: TypeAlias = "altair.vegalite.v5.api.ChartType"
@@ -96,9 +100,13 @@ def _using_vegafusion() -> bool:
 
 
 def _filter_dataframe(
-    native_df: IntoDataFrame, selection: ChartSelection
-) -> IntoDataFrame:
-    df = nw.from_native(native_df)
+    native_df: Union[IntoDataFrame, IntoLazyFrame], selection: ChartSelection
+) -> Union[IntoDataFrame, IntoLazyFrame]:
+    # Use lazy evaluation for efficient chained filtering
+    base = nw.from_native(native_df)
+    is_lazy = is_narwhals_lazyframe(base)
+    df = base.lazy()
+
     if not isinstance(selection, dict):
         raise TypeError("Input 'selection' must be a dictionary")
 
@@ -119,10 +127,12 @@ def _filter_dataframe(
             vgsid = fields.get("_vgsid_", [])
             try:
                 indexes = [int(i) - 1 for i in vgsid]
-                df = cast(nw.DataFrame[Any], df)[indexes]
+                # Need to collect for index-based selection
+                non_lazy = df.collect()[indexes]
+                df = non_lazy.lazy()
             except IndexError:
                 # Out of bounds index, return empty dataframe if it's the
-                df = cast(nw.DataFrame[Any], df)[[]]
+                df = df.head(0)
                 LOGGER.error(f"Invalid index in selection: {vgsid}")
             except ValueError:
                 LOGGER.error(f"Invalid index in selection: {vgsid}")
@@ -138,10 +148,12 @@ def _filter_dataframe(
             if field in ("vlPoint", "_vgsid_"):
                 continue
 
-            if field not in df.columns:
+            # Need to collect schema to check columns and dtypes
+            schema = df.collect_schema()
+            if field not in schema.names():
                 raise ValueError(f"Field '{field}' not found in DataFrame")
 
-            dtype = df[field].dtype
+            dtype = schema[field]
             resolved_values = _resolve_values(values, dtype)
             if is_point_selection:
                 df = df.filter(nw.col(field).is_in(resolved_values))
@@ -166,7 +178,11 @@ def _filter_dataframe(
                     f"Invalid selection: {field}={resolved_values}"
                 )
 
-    return nw.to_native(df)
+    if not is_lazy and is_narwhals_lazyframe(df):
+        # Undo the lazy
+        return df.collect().to_native()  # type: ignore[no-any-return]
+
+    return df.to_native()
 
 
 def _resolve_values(values: Any, dtype: Any) -> list[Any]:
@@ -182,14 +198,9 @@ def _resolve_values(values: Any, dtype: Any) -> list[Any]:
         if nw.Datetime == dtype and isinstance(dtype, nw.Datetime):
             if isinstance(value, str):
                 res = datetime.datetime.fromisoformat(value)
-                # If dtype has no timezone, shift by local timezone offset
-                if dtype.time_zone is None:
-                    local_tz = datetime.datetime.now().astimezone().tzinfo
-                    LOGGER.warning(
-                        f"Datetime was given with a timezone when not expected. "
-                        f"Shifting by local timezone offset {local_tz}."
-                    )
-                    return res.astimezone(local_tz).replace(tzinfo=None)
+                # If dtype has no timezone, but value has timezone, remove timezone without shifting
+                if dtype.time_zone is None and res.tzinfo is not None:
+                    return res.replace(tzinfo=None)
                 return res
 
             # Value is milliseconds since epoch
@@ -601,6 +612,13 @@ def maybe_make_full_width(chart: AltairChartType) -> AltairChartType:
             isinstance(chart, (alt.Chart, alt.LayerChart))
             and chart.width is alt.Undefined
         ):
+            # Don't make full width if chart has column encoding (faceted)
+            if (
+                hasattr(chart, "encoding")
+                and hasattr(chart.encoding, "column")
+                and chart.encoding.column is not alt.Undefined
+            ):
+                return chart
             return chart.properties(width="container")
         return chart
     except Exception:
