@@ -1,4 +1,6 @@
 # Copyright 2025 Marimo. All rights reserved.
+from __future__ import annotations
+
 from enum import IntEnum
 
 
@@ -6,4 +8,5 @@ class WebSocketCodes(IntEnum):
     ALREADY_CONNECTED = 1003
     NORMAL_CLOSE = 1000
     FORBIDDEN = 1008
+    UNAUTHORIZED = 3000
     UNEXPECTED_ERROR = 1011
